@@ -1,13 +1,13 @@
 import atexit
 
 from copy      import deepcopy
+from functools import reduce
 from sys       import getrefcount
 from os        import close
 from os        import remove
 from os        import rmdir
 from os.path   import isfile
 from operator  import mul
-from itertools import izip
 from itertools import product as itertools_product
 from tempfile  import mkstemp
 
@@ -28,12 +28,17 @@ from numpy.ma import nomask      as numpy_ma_nomask
 
 from numpy.ma.core import MaskedConstant as numpy_ma_core_MaskedConstant
 
+#from cfunits import Units
+
 from ..units     import Units
 from ..functions import get_subspace, FREE_MEMORY, FM_THRESHOLD
 from ..functions import inspect as cf_inspect
 from ..constants import CONSTANTS
 
-from .filearray import FileArray, _TempFileArray #, SharedMemoryArray, _shared_memory_array
+#from .filearray import  _TempFileArray #, SharedMemoryArray, _shared_memory_array,FileArray
+from .cachedarray import  CachedArray
+
+from .abstract import FileArray
 
 _Units_conform = Units.conform
 
@@ -80,10 +85,6 @@ not recommended to be used as a general tidy-up function.
 
 .. seealso:: `__del__`
 
-:Examples 1:
-
->>> _remove_temporary_files()
-
 :Parameters:
 
     filename: `str`, optional
@@ -95,7 +96,9 @@ not recommended to be used as a general tidy-up function.
 
     `None`
 
-:Examples 2:
+**Examples:**
+
+>>> _remove_temporary_files()
 
 >>> _temporary_files
 {'/tmp/cf_array_7hoQk5/cf_array_RmC7NJ.npy': ('/tmp/cf_array_7hoQk5',
@@ -146,20 +149,20 @@ not recommended to be used as a general tidy-up function.
             remove(filename)
         except OSError:
             pass
-        _dirname, _lock_file, _other_lock_files = _temporary_files[filename]
+        dirname, _lock_file, _other_lock_files = _temporary_files[filename]
         try:
             remove(_lock_file)
         except OSError:
             pass
-        for lockfile in _other_lock_files:
+        for lock_file in _other_lock_files:
             try:
-                remove(lockfile)
+                remove(lock_file)
             except OSError:
                 pass
         #--- End: for
         try:
             rmdir(dirname)
-        except:
+        except OSError:
             pass
     #--- End: for
 
@@ -178,13 +181,8 @@ atexit.register(_remove_temporary_files)
 # --------------------------------------------------------------------
 _copy = numpy_vectorize(deepcopy, otypes=[object])
 
-# ====================================================================
-#
-# Partition objectg
-#
-# ====================================================================
 
-class Partition(object):
+class Partition:
     '''
 
 A partition of a master data array.
@@ -237,7 +235,7 @@ data array
     Units : Units, optional
         The units of the partition's subarray. DO NOT UPDATE INPLACE.
 
-:Examples:
+**Examples:**
 
 >>> p = Partition(subarray   = numpy.arange(20).reshape(2,5,1),
 ...               location   = [(0, 6), (1, 3), (4, 5)],
@@ -324,7 +322,11 @@ by any other partition then the file is closed.
         else:
             try:
                 if (FileArray is not None and isinstance(subarray, FileArray)):
-                    filename = getattr(subarray, 'file', None)
+                    try:
+                        filename = subarray.get_filename()
+                    except:
+                        filename = None
+
                     if self.file_counter.get(filename, 999) <= 0:
                         # This partition contains a non-temporary file
                         # which is not referenced by any other
@@ -348,10 +350,10 @@ by any other partition then the file is closed.
 #
 #:Returns:
 #
-#    out : dict
+#    `dict`
 #        A dictionary of the instance's attributes
 #
-#:Examples:
+#**Examples:**
 #
 #'''
 #        return dict([(attr, getattr(self, attr))
@@ -373,7 +375,7 @@ by any other partition then the file is closed.
 #    None
 #
 #'''
-#        for attr, value in odict.iteritems():
+#        for attr, value in odict.items():
 #            setattr(self, attr, value)
 #    #--- End: def
 
@@ -410,8 +412,12 @@ temporary FileArray.
         
         try:
             if (isinstance(subarray, FileArray) and
-                not isinstance(subarray, _TempFileArray)):
-                filename = getattr(subarray, 'file', None)
+                not isinstance(subarray, CachedArray)):
+                try:
+                    filename = subarray.get_filename()
+                except:
+                    filename = None
+                    
                 if filename is None:
                     return
 
@@ -461,10 +467,6 @@ instance of FileArray and not a temporary FileArray.
 
 Assumes that ``self.config`` already exists.
 
-:Examples 1:
-
->>> p._configure_auxiliary_mask([mask_component1, mask_component2])
-
 :Parameters:
 
     auxiliary_mask: `list` of `cf.Data`
@@ -473,6 +475,10 @@ Assumes that ``self.config`` already exists.
 
     `None`
 
+**:Examples:**
+
+>>> p._configure_auxiliary_mask([mask_component1, mask_component2])
+
         '''
         indices = self.indices
                 
@@ -501,11 +507,11 @@ data array.
 
 :Returns:
 
-    out : tuple
+    `tuple`
         A tuple of slice objects or, if the master data array is a
         scalar array, an empty tuple.
 
-:Examples:
+**Examples:**
 
 >>> p.location
 [(0, 5), (2, 9)]
@@ -528,7 +534,7 @@ data array.
 True if and only if the partition's subarray is in memory as opposed
 to on disk.
 
-:Examples:
+**Examples:**
 
 >>> p.in_memory
 False
@@ -545,7 +551,7 @@ False
 #
 #.. seealso:: `array`, `in_memory`, `on_disk`, `to_shared_memory`
 #
-#:Examples:
+#**Examples:**
 #
 #>>> p.in_shared_memory
 #True
@@ -561,18 +567,18 @@ False
     # Attribute: read only
     # ----------------------------------------------------------------
     @property
-    def in_temporary_file(self):
+    def in_cached_file(self):
         '''True if and only if the partition's subarray is on disk in a
 temporary file.
 
 .. seealso:: `array`, `in_memory`, `in_shared_memory`, `on_disk`, `to_disk`
 
-:Examples:
+**Examples:**
 
->>> p.in_temporary_file
+>>> p.in_cached_file
 False
         '''
-        return isinstance(self._subarray, _TempFileArray)
+        return isinstance(self._subarray, CachedArray)
     #--- End: def
 
     # ----------------------------------------------------------------
@@ -585,7 +591,7 @@ False
 True if and only if the partition's subarray is on disk as opposed to
 in memory.
 
-:Examples:
+**Examples:**
 
 >>> p.on_disk
 True
@@ -605,7 +611,7 @@ False
         '''True if and only if the partition's subarray is on disk as opposed to
 in memory.
 
-:Examples:
+**Examples:**
 
 >>> p.on_disk
 True
@@ -614,7 +620,7 @@ True
 False
 
         '''
-        return self.on_disk and not self.in_temporary_file
+        return self.on_disk and not self.in_cached_file
     #--- End: if
 
     @property
@@ -632,7 +638,7 @@ False
 
 True if and only if the partition's data array is a scalar array.
 
-:Examples:
+**Examples:**
 
 >>> p.axes
 []
@@ -661,7 +667,7 @@ one.
         if dtype is None:
             return None
         
-        size = long(reduce(mul, self.shape, 1))
+        size = reduce(mul, self.shape, 1)
         nbytes = size * dtype.itemsize
 
         if getattr(self, 'masked', True):
@@ -684,7 +690,7 @@ one.
 
 Number of elements in the partition's data array (not its subarray).
 
-:Examples:
+**Examples:**
 
 >>> p.shape
 (73, 48)
@@ -692,7 +698,7 @@ Number of elements in the partition's data array (not its subarray).
 3504
 
 '''
-        return long(reduce(mul, self.shape, 1))
+        return reduce(mul, self.shape, 1)
     #--- End: def
 
     @property
@@ -725,7 +731,7 @@ Number of elements in the partition's data array (not its subarray).
 #
 #True if and only if the partition's subarray is in an external file.
 #
-#:Examples:
+#**Examples:**
 #
 #>>> p.subarray_in_external_file
 #False
@@ -752,7 +758,7 @@ order.
 
     None
 
-:Examples:
+**Examples:**
 
 >>> p.axes
 ['dim0', 'dim1']
@@ -808,7 +814,7 @@ values of the partition's `!_original` attribute and on the
 
     None
 
-:Examples:
+**Examples:**
 
 >>> p.array(...)
 >>> p.close()
@@ -818,7 +824,7 @@ values of the partition's `!_original` attribute and on the
         
         if config is None:
             return
-#        print 'close:', hasattr(self, 'config'),
+
         if kwargs:
             config.update(kwargs)
 
@@ -826,7 +832,7 @@ values of the partition's `!_original` attribute and on the
 
         original = getattr(self, '_original', None)
         if _debug:
-            print 'Partition.close: original =', original
+            print('Partition.close: original =', original)
         
         if not original:
             originally_on_disk = False
@@ -837,22 +843,22 @@ values of the partition's `!_original` attribute and on the
             
         config = self.config
         if _debug:
-            print ' config =',config
+            print(' config =',config)
 
         if config['serial']:
             # --------------------------------------------------------
             # SERIAL
             # --------------------------------------------------------
             if _debug:
-                print '  serial'
+                print('  serial')
 
             if config['readonly']:
                 if _debug:
-                    print '   readonly=True'
+                    print('   readonly=True')
                         
                 if originally_on_disk:
                     if _debug:
-                        print '    subarray originally on disk'
+                        print('    subarray originally on disk')
                         
                     if config.get('to_disk', False):
                         # 1.1.1.1 The original subarray was on disk,
@@ -861,7 +867,7 @@ values of the partition's `!_original` attribute and on the
                         #         to discard any changes that may have
                         #         been made to the subaray.
                         if _debug:
-                            print '    1.1.1.1 revert'
+                            print('    1.1.1.1 revert')
                         self.revert()
                     elif FREE_MEMORY() <= FM_THRESHOLD():
                         # 1.1.1.2 The original subarray was on disk,
@@ -869,14 +875,14 @@ values of the partition's `!_original` attribute and on the
                         #         subarray in memory, but there is not
                         #         enough free memory to do so.
                         if _debug:
-                            print '    1.1.1.2 revert ({} <= {})'.format(FREE_MEMORY(), FM_THRESHOLD())
+                            print('    1.1.1.2 revert ({} <= {})'.format(FREE_MEMORY(), FM_THRESHOLD()))
                         self.revert()
                     else:
                         # 1.1.1.3 The original subarray was on disk
                         #         and there is enough memory to keep
                         #         the current subarray in memory
                         if (config['unique_subarray'] and 
-                            isinstance(original_subarray, _TempFileArray)):
+                            isinstance(original_subarray, CachedArray)):
                             # The original subarray was a temporary
                             # file which is not referenced by any
                             # other partitions
@@ -884,31 +890,31 @@ values of the partition's `!_original` attribute and on the
 
                         del self.masked      
                         if _debug:
-                            print '    1.1.1.3 del masked ({} > {})'.format(FREE_MEMORY(), FM_THRESHOLD())
+                            print('    1.1.1.3 del masked ({} > {})'.format(FREE_MEMORY(), FM_THRESHOLD()))
 
                 else:
                     if _debug:
-                        print '   subarray originally in memory'
+                        print('   subarray originally in memory')
                     if config.get('to_disk', False):
                         # 1.1.2.1 Original subarray was in memory and
                         #         we don't want to keep the current
                         #         subarray in memory
                         if _debug:
-                            print '    1.1.2.1 to_disk'
+                            print('    1.1.2.1 to_disk')
                         self.to_disk(reopen=False)
                     elif FREE_MEMORY() <= FM_THRESHOLD():
                         # 1.1.2.2 Original subarray was in memory and
                         #         unique but there is not enough
                         #         memory to keep the current subarray
                         if _debug:
-                            print '    1.1.2.2 to_disk'
+                            print('    1.1.2.2 to_disk')
                         self.to_disk(reopen=False)
                     else:
                         # 1.1.2.3 Original subarray was in memory and
                         #         unique and there is enough memory to
                         #         keep the current subarray in memory
                         if _debug:
-                            print '    1.1.2.3 pass'
+                            print('    1.1.2.3 pass')
                         pass
             else:
                 # config['readonly'] is False
@@ -918,34 +924,34 @@ values of the partition's `!_original` attribute and on the
                         #         there and we don't want to keep the
                         #         array
                         if (config['unique_subarray'] and
-                            isinstance(original_subarray, _TempFileArray)):
+                            isinstance(original_subarray, CachedArray)):
                             # Original subarray was a temporary file
                             # on disk which is not referenced by any
                             # other partitions
                             _remove_temporary_files(original_subarray._partition_file)
 
                         if _debug:
-                            print '    1.2.1.1 to_disk'
+                            print('    1.2.1.1 to_disk')
                         self.to_disk(reopen=False)
                     elif FREE_MEMORY() <= FM_THRESHOLD():
                         # 1.2.1.2 Original subarray was on disk but
                         #         there is not enough memory to keep
                         #         it
                         if (config['unique_subarray'] and
-                            isinstance(original_subarray, _TempFileArray)):
+                            isinstance(original_subarray, CachedArray)):
                             # Original subarray was a temporary file
                             # on disk which is not referenced by any
                             # other partitions
                             _remove_temporary_files(original_subarray._partition_file)
 
                         if _debug:
-                            print '    1.2.1.2 to_disk'
+                            print('    1.2.1.2 to_disk')
                         self.to_disk(reopen=False)
                     else:
                         # 1.2.1.3 Original subarray was on disk and
                         #         there is enough memory to keep it
                         if _debug:
-                            print '    1.2.1.3 pass'
+                            print('    1.2.1.3 pass')
                         del self.masked       
                         pass
                 else:
@@ -953,24 +959,24 @@ values of the partition's `!_original` attribute and on the
                         # 1.2.2.1 Original subarray was in memory but
                         #         we don't want to keep it
                         if _debug:
-                            print '    1.2.2.1 to_disk'
+                            print('    1.2.2.1 to_disk')
                         self.to_disk(reopen=False)
                     elif FREE_MEMORY() <= FM_THRESHOLD():
                         # 1.2.2.2 Original subarray was an in memory
                         #         but there is not enough memory to
                         #         keep it
                         if _debug:
-                            print '    1.2.2.2 to_disk'
+                            print('    1.2.2.2 to_disk')
                         self.to_disk(reopen=False)
                     else:
                         # 1.2.2.3 Original subarray was in memory and
                         #         there is enough memory to keep it
                         if _debug:
-                            print '    1.2.2.3 del masked'
+                            print('    1.2.2.3 del masked')
                         del self.masked       
         else:
             if _debug:
-                print 'Partition.close: parallel'
+                print('Partition.close: parallel')
             # --------------------------------------------------------
             # PARALLEL
             # --------------------------------------------------------
@@ -988,59 +994,52 @@ values of the partition's `!_original` attribute and on the
     #--- End: def
 
     def copy(self):
+        '''Return a deep copy.
+
+    ``p.copy()`` is equivalent to ``copy.deepcopy(p)``.
+    
+    :Returns:
+    
+            A deep copy.
+    
+    **Examples:**
+    
+    >>> q = p.copy()
+
         '''
-
-Return a deep copy.
-
-``p.copy()`` is equivalent to ``copy.deepcopy(p)``.
-
-:Returns:
-
-    out :
-        A deep copy.
-
-:Examples:
-
->>> q = p.copy()
-
-'''
         new = Partition.__new__(Partition)
         new.__dict__ = self.__dict__.copy()
 
         self._increment_file_counter()
         
         return new
-    #--- End: def
+
 
     @property
     def array(self):
-        '''
-
-Returns the partition's data array.
-
+        '''Returns the partition's data array.
     
-After a partition has been conformed, the partition must be closed
-(with the `close` method) before another partition is conformed,
-otherwise a memory leak could occur. For example:
+    After a partition has been conformed, the partition must be closed
+    (with the `close` method) before another partition is conformed,
+    otherwise a memory leak could occur. For example:
+    
+    >>> for partition in partition_array.flat:
+    ...    # Open the partition
+    ...    partition.open(config)
+    ...
+    ...    # Get the data array as a numpy array
+    ...    array = partition.array
+    ...
+    ...    # < Some code to operate on the aray >
+    ...
+    ...    # Close the partition
+    ...    partition.close()
+    ...
+    ...    # Now move on to conform the next partition 
+    ...
+    >>>
 
->>> for partition in partition_array.flat:
-...
-...    # Open the partition
-...    partition.open(config)
-...
-...    # Get the data array as a numpy array
-...    array = partition.array
-...
-...    # < Some code to operate on the aray >
-...
-...    # Close the partition
-...    partition.close()
-...
-...    # Now move on to conform the next partition 
-...
->>>  
-
-'''
+        '''
         config = self.config
 
         unique_array = config['unique_subarray']
@@ -1050,7 +1049,6 @@ otherwise a memory leak could occur. For example:
         p_part   = self.part
         p_units  = self.Units
         p_shape  = self.shape
-        p_shape  = self.shape
         p_location = self.location
         subarray = self._subarray
 
@@ -1172,7 +1170,6 @@ otherwise a memory leak could occur. For example:
                             else slice(None))
                            for axis in p_axes]
                 p_data = p_data[tuple(indices)]
-            #--- End: if
     
             # --------------------------------------------------------
             # Transpose axes
@@ -1181,9 +1178,10 @@ otherwise a memory leak could occur. For example:
                 iaxes = [p_axes.index(axis) for axis in axes if axis in p_axes]
                 
                 if len_p_axes > len(iaxes):
-                    for i in xrange(len_p_axes):
+                    for i in range(len_p_axes):
                         if i not in iaxes:
-                            iaxes.append(i)
+#                            iaxes.append(i)
+                            iaxes.insert(i, i)
                 #--- End: if
                 
                 p_data = numpy_transpose(p_data, iaxes)
@@ -1192,7 +1190,9 @@ otherwise a memory leak could occur. For example:
         # ------------------------------------------------------------
         # Remove excessive/insert missing size 1 axes
         # ------------------------------------------------------------
-        if len_p_axes != len(p_shape):
+        # print ('p_shape=', p_shape, p_axes, p_data.shape)
+        if p_shape != p_data.shape:
+#        if len_p_axes != len(p_shape):
             p_data = p_data.reshape(p_shape)
 
         # ------------------------------------------------------------
@@ -1208,8 +1208,8 @@ otherwise a memory leak could occur. For example:
  
                     p_data.mask = (mask | p_data.mask).array
             #--- End: for
+            
             self.masked = True
-        #--- End: if
 
         # ------------------------------------------------------------
         # Convert the array's data type
@@ -1221,7 +1221,8 @@ otherwise a memory leak could occur. For example:
                 p_data = p_data.astype(dtype) # Note: returns a copy
             except ValueError:
                 raise ValueError(
-"Can't recast partition array from {} to {}".format(p_dtype.name, dtype.name))
+                    "Can't recast partition array from {} to {}".format(
+                        p_dtype.name, dtype.name))
             else:
                 # We've just copied p_data, so in place changes are
                 # not possible
@@ -1264,32 +1265,29 @@ otherwise a memory leak could occur. For example:
             self.location  = p_location
 
             self._in_place_changes = in_place_changes
-        #--- End: if
 
         # ------------------------------------------------------------
         # Return the numpy array
         # ------------------------------------------------------------
         return p_data
-    #--- End: def
+
     
     @property
     def isdt(self):
+        '''True if the subarray contains date-time objects.
+
+    **Examples:**
+    
+    >>> p.Units.isreftime
+    True
+    >>> p.subarray.dtype == numpy.dtype(object)
+    True
+    >>> p.isdt
+    True
+
         '''
-
-True if the subarray contains date-time objects.
-
-:Examples:
-
->>> p.Units.isreftime
-True
->>> p.subarray.dtype == numpy.dtype(object)
-True
->>> p.isdt
-True
-
-'''
         return self.Units.isreftime and self._subarray.dtype == _dtype_object
-    #--- End: def
+
 
     def file_close(self):
         '''
@@ -1300,7 +1298,7 @@ Close all file containing the subarray, if there is one.
 
     None
 
-:Examples:
+**Examples:**
 
 >>> p.file_close()
 
@@ -1319,10 +1317,10 @@ Close all file containing the subarray, if there is one.
 #
 #:Returns:
 #
-#    out : generator
+#    generator
 #        An iterator that yields the partition itself.
 #
-#:Examples:
+#**Examples:**
 #
 #>>> type(p.flat())
 #<generator object flat at 0x519a0a0>
@@ -1345,10 +1343,10 @@ Close all file containing the subarray, if there is one.
 #
 #:Returns:
 #
-#    out : generator
+#    generator
 #        An iterator over indices of the partition's data array.
 #
-#:Examples:
+#**Examples:**
 #
 #>>> p.shape
 #[2, 1, 3]
@@ -1370,7 +1368,7 @@ Close all file containing the subarray, if there is one.
 #()
 #
 #'''
-#        return itertools_product(*[xrange(0, r) for r in self.shape])
+#        return itertools_product(*[range(0, r) for r in self.shape])
 #    #--- End: def
 
     def inspect(self):
@@ -1384,12 +1382,12 @@ Inspect the object for debugging.
 
     None
 
-:Examples:
+**Examples:**
 
 >>> f.inspect()
 
 '''
-        print cf_inspect(self)
+        print(cf_inspect(self))
     #--- End: def
 
     def master_ndindex(self): #itermaster_indices(self):
@@ -1400,11 +1398,11 @@ by the data array.
 
 :Returns:
 
-    out : generator
+    generator
         An iterator over indices of the master array which are spanned
         by the data array.
 
-:Examples:
+**Examples:**
 
 >>> p.location
 [(3, 5), (0, 1), (0, 3)]
@@ -1419,7 +1417,7 @@ by the data array.
 (4, 0, 2)
 
 '''
-        return itertools_product(*[xrange(*r) for r in self.location])
+        return itertools_product(*[range(*r) for r in self.location]) # TODO check
     #--- End: def
 
     def new_part(self, indices, master_axis_to_position, master_flip):
@@ -1440,7 +1438,7 @@ array.
 
     None
 
-:Examples:
+**Examples:**
 
 >>> p.new_part(indices, dim2position, master_flip)
 
@@ -1474,7 +1472,7 @@ array.
 
             p_flip = self.flip
   
-            for axis, i in master_axis_to_position.iteritems():
+            for axis, i in master_axis_to_position.items():
                       
                 if ((axis not in p_flip and axis not in master_flip) or 
                     (axis in p_flip     and axis in master_flip)):
@@ -1528,9 +1526,9 @@ array.
 
         # Still here? update an existing part
         p_part = []
-        for part_index, index, size in izip(part,
-                                            indices, 
-                                            self._subarray.shape):
+        for part_index, index, size in zip(part,
+                                           indices, 
+                                           self._subarray.shape):
 
             if index == slice_None:
                 p_part.append(part_index)
@@ -1564,7 +1562,7 @@ array.
 
                     continue
                 else:
-                    new_part = range(*part_index.indices(size))
+                    new_part = list(range(*part_index.indices(size)))
                     new_part = [new_part[i] for i in index]
             else:
                 if isinstance(index, slice):
@@ -1587,7 +1585,7 @@ array.
                         start, stop = new_part0, new_part[-1]+1
                     else:
                         start, stop = new_part0, new_part[-1]-1
-                        if new_part == range(start, stop, step):
+                        if new_part == list(range(start, stop, step)):
                             if stop < 0:
                                 stop = None
                             new_part = slice(start, stop, step)
@@ -1716,7 +1714,7 @@ array.
         overlaps the master indices and the subsetted partition's
         shape as a list. Otherwise return `None`, `None`.
 
-:Examples:
+**Examples:**
 
 >>> indices = (slice(None), slice(5, 1, -2), [1, 3, 4, 8])
 >>> p.overlaps(indices)
@@ -1729,7 +1727,7 @@ array.
         if not indices:
             return p_indices, shape
 
-        for index, (r0, r1), size in izip(indices, self.location, self.shape):
+        for index, (r0, r1), size in zip(indices, self.location, self.shape):
             if isinstance(index, slice):
                 stop = size
                 if index.stop < r1:
@@ -1766,7 +1764,7 @@ array.
                         start, stop = index0, index[-1]+1
                     elif step < 0:
                         start, stop = index0, index[-1]-1
-                    if index == range(start, stop, step):
+                    if index == list(range(start, stop, step)):
                         # Replace the list with a slice object
                         if stop < 0:
                             stop = None
@@ -1816,19 +1814,19 @@ memory, but this is not checked.
 
 :Returns:
 
-    out: `bool`
+    `bool`
         True if the subarray was moved to disk, False otherwise.
 
-:Examples:
+**Examples:**
 
 >>> p.to_disk()
 >>> p.to_disk(reopen=False)
 
         '''
-        try:
-            tfa = _TempFileArray(self.array)
-        except:
-            return False
+#        try:
+        tfa = CachedArray(self.array)
+#        except:
+#            return False
 
         fd, _lock_file = mkstemp(prefix=tfa._partition_file + '_',
                                  dir=tfa._partition_dir)
@@ -1854,7 +1852,7 @@ memory, but this is not checked.
 #
 #:Returns:
 #
-#    out : bool
+#    `bool`
 #        Whether or not the subarray is in shared memory.
 #
 #'''
@@ -1896,7 +1894,7 @@ The updated partition is always dependent of the other partition.
 
     `None`
 
-:Examples:
+**Examples:**
 
 >>> p.revert()
 
@@ -1920,24 +1918,22 @@ The updated partition is always dependent of the other partition.
     #--- End: def
 
     def update_inplace_from(self, other):
-        '''
-
-Completely update the partition with another partition's attributes in
+        '''Completely update the partition with another partition's attributes in
 place.
 
 :Parameters:
 
-    other : Partition
+    other: `Partition`
 
 :Returns:
 
-    None
+    `None`
 
-:Examples:
+**Examples:**
 
 >>> p.update_inplace_from(q)
 
-'''
+        '''
         self.__dict__ = other.__dict__.copy()
     #--- End: def
 
@@ -1948,7 +1944,7 @@ place.
     #         # If BOTH subarrays are in shared memory
     #         _shared_memory_array[id(other.subarray)] = _shared_memory_array[id(self.subarray)] 
             
-    #     if other.in_temporary_file:
+    #     if other.in_cached_file:
     #         # The other subarray is in a temporary file
     #         _temporary_files.add(other._subarray._partition_file)
             
@@ -1960,7 +1956,7 @@ place.
         # on another rank
         _partition_file = self._subarray._partition_file
         _partition_dir = self._subarray._partition_dir
-        if not _partition_file in _temporary_files:
+        if _partition_file not in _temporary_files:
             fd, _lock_file = mkstemp(prefix=_partition_file + '_',
                                      dir=_partition_dir)
             close(fd)
